package org.xdoclet.plugin.beaninfo;

import com.thoughtworks.qdox.model.DocletTag;
import com.thoughtworks.qdox.model.JavaClass;
import com.thoughtworks.qdox.model.JavaMethod;
import com.thoughtworks.qdox.model.JavaParameter;
import com.thoughtworks.qdox.model.Type;
import org.generama.VelocityTemplateEngine;
import org.generama.WriterMapper;
import org.generama.defaults.JavaGeneratingPlugin;
import org.xdoclet.QDoxMetadataProvider;
import org.xdoclet.plugin.beaninfo.qtags.JavabeanClassTagImpl;

import java.util.HashMap;
/**
 * @author Laurent Etiemble
 * @author Aslak Helles&oslash;y
 * @version $Revision: 1.9 $
 */
public class BeanInfoPlugin extends JavaGeneratingPlugin {
    public BeanInfoPlugin(VelocityTemplateEngine velocityTemplateEngine, QDoxMetadataProvider metadataProvider, WriterMapper writerMapper) {
        super(velocityTemplateEngine, metadataProvider, writerMapper);

        setMultioutput(true);

        setFileregex("\\.java");
        setFilereplace("BeanInfo.java");

        metadataProvider.getDocletTagFactory().registerTag(JavabeanClassTagImpl.NAME, JavabeanClassTagImpl.class);
    }

    public boolean shouldGenerate(Object metadata) {
        JavaClass javaClass = (JavaClass) metadata;
        DocletTag javaBeanTag = javaClass.getTagByName("javabean.class");
        return (javaBeanTag != null);
    }

    // Callback methods from Velocity Template

    // ========================================
    // Maybe they will be moved as they are generic
    // ========================================

    public String getParameterTypes(JavaMethod method) {
        StringBuffer result = new StringBuffer();
        JavaParameter[] params = method.getParameters();
        for (int i = 0; i < params.length; i++) {
            if (i > 0) {
                result.append(", ");
            }
            result.append(typeToClass(params[i].getType()));
        }
        return result.toString();
    }

    private String typeToClass(Type type) {
        String result = type.getValue();
		if(primitiveMap.get(result) != null) {
			return (String)primitiveMap.get(result);
		}
		
		return result + ".class";
    }
	
	private static final HashMap primitiveMap = new HashMap();
	static {
		primitiveMap.put("boolean","Boolean.TYPE");
		primitiveMap.put("int","Integer.TYPE");
		primitiveMap.put("long","Long.TYPE");
		primitiveMap.put("short","Short.TYPE");
		primitiveMap.put("float","Float.TYPE");
		primitiveMap.put("double","Double.TYPE");
		primitiveMap.put("byte","Byte.TYPE");
	}
}