package org.xdoclet.plugin.ejb;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.collections.Predicate;
import com.thoughtworks.qdox.model.JavaClass;
import com.thoughtworks.qdox.model.JavaMethod;

import java.util.Collection;
import java.util.Arrays;

/**
 * This class holds common functionality used by all EJB plugin.
 * These methods will mostly be called by templates (including
 * vendor specific templates). Reusable code should be placed here.
 *
 * @author Aslak Helles&oslash;y
 * @version $Revision: 1.3 $
 */
public class EjbUtils {

    public EjbUtils() throws ClassNotFoundException {
// Try to load one of the javax.ejb classes so the user can
// get an early warning if she has forgotten to stick it on the classpath
        getClass().getClassLoader().loadClass("javax.ejb.EntityBean");
    }

    public boolean shouldGenerate(Object metadata) {
        JavaClass javaClass = (JavaClass) metadata;
        boolean isEjb = javaClass.isA("javax.ejb.EntityBean");
        boolean ignore = "false".equals(javaClass.getNamedParameter("ejb.bean", "generate"));
        return isEjb && !ignore;
    }

    public String getEjbName(JavaClass clazz) {
        String result = clazz.getNamedParameter("ejb.bean", "name");
        if (result == null) {
            result = clazz.getName();
        }
        return result;
    }

    /**
     * Returns interface methods on the bean
     * @return Collection of {@link com.thoughtworks.qdox.model.JavaMethod}
     */
    public Collection getInterfaceMethods(final JavaClass clazz, final String desiredViewType) {
        if (clazz == null || desiredViewType == null) throw new Error();
        return CollectionUtils.select(Arrays.asList(clazz.getMethods()), new Predicate() {
            public boolean evaluate(Object object) {
                JavaMethod method = (JavaMethod) object;
                String viewType = method.getNamedParameter("ejb.interface-method", "view-type");
                if (viewType == null) {
                    viewType = clazz.getNamedParameter("ejb.bean", "view-type");
                }

                if (viewType != null) {
                    boolean isLocal = "local".equals(viewType) || "both".equals(viewType);
                    boolean isRemote = "remote".equals(viewType) || "both".equals(viewType);
                    if (desiredViewType.equals("local")) {
                        return isLocal;
                    }
                    if (desiredViewType.equals("remote")) {
                        return isRemote;
                    }
                }
                return false;
            }
        });
    }
}
