package org.xdoclet.plugin.ejb.qtags;

/**
 * Defines the data access object of an entity bean. If the <dao/> subtask is
 * included, then DAO interfaces will be generated for any entity beans
 * containing this tag, unless a generate="false" parameter is present.
 */
public interface EjbDaoTag extends com.thoughtworks.qdox.model.DocletTag {
    /**
     * The fully qualified name of the DAO interface. If
     * absent the interface name will be determined using
     * the pattern and package parameters.
     *
     */
    String getClass_();

    /**
     * Define the pattern (using {0} for the EJB name) to
     * be used for constructing the unqualified (package
     * specific) name of the DAO interface.
     *
     */
    String getPattern();

    /**
     * Define the package the DAO interface is in
     * (generated or not). Use <packageSubstitution/> to
     * configure which package the interface should end
     * up in.
     *
     */
    String getPackage();

    /**
     * Indicate if the DAO interface should be generated.
     * Defaults to true.
     *
     */
    boolean isGenerate();

    /**
     * The fully qualified name of a class which
     * implements the DAO interface. If present, then
     * generated code for obtaining a DAO will just
     * create a new instance of this class.
     *
     */
    String getImplClass();

    /**
     * Indicates a JNDI environment reference which will
     * contain a String that is the fully qualified name of
     * a class implementing the DAO interface. If present,
     * then generated code for obtaining a DAO will look
     * up this reference, then create a new instance of the
     * class whose name is found there. This allows the
     * actual implementation class to be determined at
     * runtime.
     *
     */
    String getImplJndi();

    /**
     * Indicate if create methods should be automatically
     * included in the DAO interface. Defaults to true.
     *
     */
    String getCreateMethods();

    /**
     * Indicate if finder methods should be automatically
     * included in the DAO interface. Defaults to true.
     *
     */
    String getFinderMethods();

}
