package org.xdoclet.plugin.ejb.qtags;

/**
 * Defines an EJB reference to a bean which will be packaged in a different
 * ejb jar file. Applicable to all types of EJBs.
 */
public interface EjbEjbExternalRefTag extends com.thoughtworks.qdox.model.DocletTag {
    /**
     * The view type of the referenced bean. Legal values are local and remote.
     *
     * @qtags.required
     * @qtags.allowed-value local
     * @qtags.allowed-value remote
     */
    String getViewType();

    /**
     * Defines an ejb-link to the referenced EJB. The value of the ejb-link
     * element is the name of the target enterprise bean. (It is the name defined in the ejb-name element
     * of the target enterprise bean.) The target enterprise bean can be in any ejb-jar file in the
     * same J2EE application as the referencing application component.
     *
     */
    String getLink();

    /**
     * The name that the referenced bean will be referred to
     * by. For example, to refer to the bean Customer as
     * java:comp/env/ejb/Customer name should be
     * ejb/Customer. Defaults to ejb/[ejb-name], where
     * [ejb-name] is the named of the referenced bean
     * (Customer) prefixed by "ejb/".
     *
     * @qtags.required
     */
    String getRefName();

    /**
     * The name that the referenced bean will be referred to
     * by. For example, to refer to the bean Customer as
     * java:comp/env/ejb/Customer name should be
     * ejb/Customer. Defaults to ejb/[ejb-name], where
     * [ejb-name] is the named of the referenced bean
     * (Customer) prefixed by "ejb/". Deprecated. Use ref-name instead.
     *
     */
    String getEjbName();

    /**
     * The type of EJB being referenced.
     *
     * @qtags.required
     * @qtags.allowed-value Entity
     * @qtags.allowed-value Session
     */
    String getType();

    /**
     * The fully qualified class name of the home interface of the referenced bean.
     *
     * @qtags.required
     */
    String getHome();

    /**
     * The fully qualified class name of the business interface (local or remote
     * interface) of the referenced bean.
     *
     * @qtags.required
     */
    String getBusiness();

    /**
     * (Deprecated). Synonym for 'business' parameter. Only works if the view-type is 'remote'
     *
     */
    String getRemote();

}
