package org.xdoclet.plugin.ejb.qtags;

/**
 * This tag is optional and allows generation of session facade EJB for
 * entity bean. Facade EJB will provide all finder and creation methods to
 * outside, and also all data setting methods for data and value objects.
 * Finder methods will return collections of data or value objects instead of
 * remote interfaces.
 * Facade bean will connect to ejb using local  ( if generated ) or remote
 * interfaces.
 * Resulting bean needs processing by xdoclet together with all ejb classes.
 */
public interface EjbFacadeTag extends com.thoughtworks.qdox.model.DocletTag {
    /**
     * Name of generated facade ejb. Default is {0}Facade
     *
     */
    String getName_();

    /**
     * A description of the bean.  Defaults to the first sentence of the
     * class-level javadoc comment of the bean class.
     *
     */
    String getDescription();

    /**
     * The display name of the bean.
     *
     */
    String getDisplayName();

    /**
     * The small icon for the bean.
     *
     */
    String getSmallIcon();

    /**
     * The large icon for the bean.
     *
     */
    String getLargeIcon();

    /**
     * Provides the JNDI name of the
     * bean that will be used in the
     * vendor specific deployment descriptors.
     *
     */
    String getJndiName();

    /**
     * Provides the JNDI name of the bean
     * that will be used in the vendor
     * specific deployment descriptors.
     * It's the JNDI name for the local EJB.
     *
     */
    String getLocalJndiName();

    /**
     * Indicates what view-type(s) is(are)
     * supported by the bean.
     *
     * @qtags.allowed-value local
     * @qtags.allowed-value remote
     * @qtags.allowed-value both
     */
    String getViewType();

    /**
     * Defines the bean's "type".
     *
     * @qtags.required
     * @qtags.allowed-value Stateless
     * @qtags.allowed-value Stateful
     */
    String getType();

    /**
     * Defines the bean's transaction type.
     *
     * @qtags.allowed-value Container
     * @qtags.allowed-value Bean
     */
    String getTransactionType();

}
