package org.xdoclet.plugin.ejb.qtags;

/**
 * Defines a finder method for the home interface. An entity bean may have zero or more
 * ejb.finder tags.
 */
public interface EjbFinderTag extends com.thoughtworks.qdox.model.DocletTag {
    /**
     * To restrict the home interface in witch the finder will be defined.
     *
     * @qtags.allowed-value local
     * @qtags.allowed-value remote
     * @qtags.allowed-value both
     */
    String getViewType();

    /**
     * Description of the finder
     *
     */
    String getDescription();

    /**
     * Defines the signature of the
     * method. E.g.. "java.util.Collection
     * findAll()". The return type in the
     * signature must be fully qualified.
     *
     * @qtags.required
     */
    String getSignature();

    /**
     * Flags the finder as having
     * unchecked permission.
     * If present, role-name must be
     * omitted
     *
     */
    boolean isUnchecked();

    /**
     * A comma-separated list of roles that can invoke this finder method.
     *
     */
    String getRoleName();

    /**
     * EJB-QL query for the finder.
     *
     */
    String getQuery();

    /**
     * Result type mapping (indicates the return type) for the finder
     * method.
     *
     * @qtags.allowed-value none
     * @qtags.allowed-value Local
     * @qtags.allowed-value Remote
     */
    String getResultTypeMapping();

    /**
     * Indicates the transaction type attribute sets to the method.
     *
     */
    String getTransactionType();

    /**
     * Indicates the interface (Home or
     * LocalHome) for which the finder
     * permissions should be set. If the
     * permissions should apply to both
     * Home or LocalHome interfaces,
     * this parameter should not be
     * specified.
     *
     * @qtags.allowed-value Home
     * @qtags.allowed-value LocalHome
     */
    String getMethodIntf();

}
