package org.xdoclet.plugin.ejb.qtags;

/**
 * The @ejb.home tag provides information about an Entity or Session
 * Bean's home interface. All parameters are applicable to both Entity and Session beans.
 */
public interface EjbHomeTag extends com.thoughtworks.qdox.model.DocletTag {
    /**
     * Declare which interface the generated
     * home interface should extend. Default is
     * "javax.ejb.EJBHome".
     *
     */
    String getExtends();

    /**
     * Declare which interface the generated
     * local home interface should extend.
     * Default is "javax.ejb.EJBLocalHome".
     *
     */
    String getLocalExtends();

    /**
     * Which home interfaces should be
     * generated. Defaults to true.
     *
     * @qtags.allowed-value true
     * @qtags.allowed-value false
     */
    boolean isGenerate();

    /**
     * The fully qualified name of the remote
     * home interface.
     *
     */
    String getRemoteClass();

    /**
     * The fully qualified name of the local
     * home interface.
     *
     */
    String getLocalClass();

    /**
     * The pattern to be used in determining the
     * unqualified name of the remote home
     * interface - only used if remote-class is
     * not present.
     *
     */
    String getRemotePattern();

    /**
     * The pattern to be used in determining the
     * unqualified name of the local home
     * interface - only used if local-class is
     * not present.
     *
     */
    String getLocalPattern();

    /**
     * The pattern to be used in determining the
     * unqualified name of either local or
     * remote home interfaces. Used where
     * -class and -pattern tags are absent for the
     * bean.
     *
     */
    String getPattern();

    /**
     * The package the remote home interface
     * should be in - only used where
     * remote-class is not present. Use
     * <packageSubstitution/> to configure
     * which package the interface should end
     * up in.
     *
     */
    String getRemotePackage();

    /**
     * The package the local home interface
     * should be in - only used where
     * local-class is not present. Use
     * <packageSubstitution/> to configure
     * which package the interface should end
     * up in.
     *
     */
    String getLocalPackage();

    /**
     * The package the  home interfaces to be
     * placed in - only used where
     * -class or -package is not present. Use
     * <packageSubstitution/> to configure
     * which package the interface should end
     * up in.
     *
     */
    String getPackage();

}
