package org.xdoclet.plugin.ejb.qtags;

/**
 * The @ejb.interface tag provides information about an Entity or
 * Session Bean's component interfaces (remote and/or local). It is not
 * applicable to Message-driven beans. All parameters are applicable to both
 * Entity and Session beans.
 */
public interface EjbInterfaceTag extends com.thoughtworks.qdox.model.DocletTag {
    /**
     * Declare which interface the generated
     * remote interface should extend. Default is
     * "javax.ejb.EJBObject".
     *
     */
    String getExtends();

    /**
     * Declare which interface the generated local
     * interface should extend. Default is
     * "javax.ejb.EJBLocalObject".
     *
     */
    String getLocalExtends();

    /**
     * Which interfaces should be generated.
     * Defaults to true.
     *
     * @qtags.allowed-value true
     * @qtags.allowed-value false
     */
    String getGenerate();

    /**
     * The fully qualified name of the remote
     * interface.
     *
     */
    String getRemoteClass();

    /**
     * The fully qualified name of the local
     * interface.
     *
     */
    String getLocalClass();

    /**
     * The pattern to be used in determining the
     * unqualified name of the remote interface -
     * only used if remote-class is not present.
     *
     */
    String getRemotePattern();

    /**
     * The pattern to be used in determining the
     * unqualified name of the local interface -
     * only used if local-class is not present.
     *
     */
    String getLocalPattern();

    /**
     * The pattern to be used in determining the
     * unqualified name of either local or remote
     * interfaces. Used where -class and -pattern
     * tags are absent for the bean.
     *
     */
    String getPattern();

    /**
     * The package the remote interface should be
     * in - only used where remote-class is not
     * present. Use <packageSubstitution/> to
     * configure which package the interface
     * should end up in.
     *
     */
    String getRemotePackage();

    /**
     * The package the local interface should be in
     * - only used where local-class is not
     * present. Use
     * <packageSubstitution/> to configure which
     * package the interface should end up in.
     *
     */
    String getLocalPackage();

    /**
     * The package for the interfaces to be placed
     * in. Only used where -class and -package
     * are not present. Use
     * <packageSubstitution/> to configure which
     * package the interface should end up in.
     *
     */
    String getPackage();

}
