package org.xdoclet.plugin.ejb.qtags;

/**
 * The @ejb.persistence tag provides information about the persistence
 * of a CMP Entity bean's fields.
 * Use this for methods with the following patterns:
 * public abstract Foo getX();
 * public abstract boolean isX();
 * public abstract void setX(Foo newVal);
 * This will create the CMP field "X" in the generated CMP layer of type
 * "Foo", and concrete implementations of the getX/setX methods. For BMP,
 * it will generate getX/setX methods that keep track of a dirty flag (so that
 * ejbStore is only called if necessary). If the type is boolean it is ok to have
 * the get method prefixed "is" instead of "get". If you don't define the setter
 * method, no setter method is generated for the concrete BMP/CMP class
 * (concrete CMP class is generated only if EJB 1.1 or EJB 2.0 but using CMP
 * 1.x). This is useful for cases where for example nothing is set
 * programmatically but via external sources such as a stored procedure.
 * Applicable for entity beans.
 */
public interface EjbPersistenceTag extends com.thoughtworks.qdox.model.DocletTag {
    /**
     * Declare the column-name to which the CMP
     * field will be mapped.
     *
     */
    String getColumnName();

    /**
     * Declare the JDBC Type. Its value must be one
     * of the fields of java.sql.Types (e.g. BIT,
     * CHAR...). This jdbc type will be used by the
     * CMP provider to determine which method to
     * call on PreparedStatement and ResultSet for
     * INSERT / UPDATE / SELECT queries.
     *
     * @qtags.allowed-value ARRAY
     * @qtags.allowed-value BIGINT
     * @qtags.allowed-value BINARY
     * @qtags.allowed-value BIT
     * @qtags.allowed-value BLOB
     * @qtags.allowed-value BOOLEAN
     * @qtags.allowed-value CHAR
     * @qtags.allowed-value CLOB
     * @qtags.allowed-value DATALINK
     * @qtags.allowed-value DATE
     * @qtags.allowed-value DECIMAL
     * @qtags.allowed-value DISTINCT
     * @qtags.allowed-value DOUBLE
     * @qtags.allowed-value FLOAT
     * @qtags.allowed-value INTEGER
     * @qtags.allowed-value JAVA_OBJECT
     * @qtags.allowed-value LONGVARBINARY
     * @qtags.allowed-value LONGVARCHAR
     * @qtags.allowed-value NULL
     * @qtags.allowed-value NUMERIC
     * @qtags.allowed-value OTHER
     * @qtags.allowed-value REAL
     * @qtags.allowed-value REF
     * @qtags.allowed-value SMALLINT
     * @qtags.allowed-value STRUCT
     * @qtags.allowed-value TIME
     * @qtags.allowed-value TIMESTAMP
     * @qtags.allowed-value TINYINT
     * @qtags.allowed-value VARBINARY
     * @qtags.allowed-value VARCHAR
     */
    String getJdbcType();

    /**
     * Declare the actual type of the field in the
     * database. This value will only be used when the
     * CMP container creates your table (if possible).
     *
     */
    String getSqlType();

    /**
     * Indicates that a field is read-only.
     *
     */
    boolean isReadOnly();

}
