package org.xdoclet.plugin.ejb.qtags;

/**
 * Defines the primary key of an entity bean. If the <entitypk/> subtask is
 * included, then primary key classes will be generated for all entity beans
 * unless a generate="false" parameter is present, or the primary class is in the
 * java.lang package.
 */
public interface EjbPkTag extends com.thoughtworks.qdox.model.DocletTag {
    /**
     * The fully qualified name of the primary key
     * class. If absent the primary key class name will
     * be determined using the pattern and package
     * parameters.
     *
     */
    String getClass_();

    /**
     * Define the pattern (using {0} for the EJB name)
     * to be used for constructing the unqualified
     * (package specific) name of the primary key
     * class.
     *
     */
    String getPattern();

    /**
     * Define the package the pk class is in (generated
     * or not). Use <packageSubstitution/> to
     * configure which package the PK class should
     * end up in.
     *
     */
    String getPackage();

    /**
     * Indicate if the primary key class should be
     * generated. Defaults to true.
     *
     */
    boolean isGenerate();

    /**
     * Define which class the generated pk must
     * extend.
     *
     */
    String getExtends();

    /**
     * Define which interface the generated pk must
     * implement.
     *
     */
    String getImplements();

    /**
     * A comma-separated list of roles that can
     * execute findByPrimaryKey
     *
     */
    String getRoleName();

    /**
     * Flags the PK-finder as having unchecked
     * permission. If present, role-name
     * must be omitted
     *
     */
    boolean isUnchecked();

    /**
     * Indicates the interface (Home or LocalHome)
     * for which the findByPrimaryKey permissions
     * should be set. Applies only if role-name or
     * unchecked is specified. If the permissions
     * should apply to both Home or LocalHome
     * interfaces, this parameter should not be
     * specified.
     *
     * @qtags.allowed-value Home
     * @qtags.allowed-value LocalHome
     */
    String getMethodIntf();

}
