package org.xdoclet.plugin.externalizer;

import com.thoughtworks.qdox.model.JavaClass;
import org.generama.Plugin;
import org.generama.VelocityTemplateEngine;
import org.generama.WriterMapper;
import org.xdoclet.QDoxMetadataProvider;
import org.xdoclet.plugin.externalizer.qtags.MsgMessageTagImpl;
import org.xdoclet.plugin.externalizer.qtags.TagLibrary;

public class MessageBundlePlugin extends Plugin {
    private String lang;
    private String country;
    private String variant;

    public MessageBundlePlugin(VelocityTemplateEngine templateEngine, QDoxMetadataProvider metadataProvider,
                               WriterMapper writerMapper) {
        super(templateEngine, metadataProvider, writerMapper);
        setFileregex("\\.java");
        setFilereplace("\\" + getBundleKey() + ".properties");
        setMultioutput(true);
        new TagLibrary(metadataProvider);
    }

    public String getLang() {
        return lang;
    }

    public String getCountry() {
        return country;
    }

    public String getVariant() {
        return variant;
    }

    public void setLang(String lang) {
        this.lang = lang;
    }

    public void setCountry(String country) {
        this.country = country;
    }

    public void setVariant(String variant) {
        this.variant = variant;
    }

    public String getBundleKey() {
        StringBuffer sb = new StringBuffer();
        if (getLang() != null) {
            sb.append(getLang());
        } else {
            return "";
        }

        if (getCountry() != null) {
            sb.append("_");
            sb.append(getCountry());
        } else {
            return sb.toString();
        }

        if (getVariant() != null) {
            sb.append("_");
            sb.append(getVariant());
        }
        return sb.toString();
    }

    /**
     * decide whether this message tag is applicable  in current context.
     * language country and variant shall match
     */
    public boolean messageApplicable(MsgMessageTagImpl tag) {
        if (getLang() == null) {
            return tag.getLang() == null;
        } else if (getLang().equals(tag.getLang())) {
            if (getCountry() == null) {
                return tag.getCountry() == null;
            } else if (getCountry().equals(tag.getCountry())) {
                if (getVariant() == null) {
                    return tag.getVariant() == null;
                } else {
                    return getVariant().equals(tag.getVariant());
                }
            }
        }
		
        // fallen through
        return false;
    }

    public boolean shouldGenerate(Object metadata) {
        JavaClass clazz = (JavaClass) metadata;
        return clazz.getTagByName("msg.bundle") != null;
    }
}
