/*
 * Copyright (c) 2003
 * XDoclet Team
 * All rights reserved.
 */
package org.xdoclet.plugin.hibernate;

import com.thoughtworks.qdox.model.JavaClass;
import org.apache.commons.collections.CollectionUtils;
import org.generama.JellyTemplateEngine;
import org.generama.Plugin;
import org.generama.WriterMapper;
import org.xdoclet.QDoxMetadataProvider;
import org.xdoclet.plugin.hibernate.qtags.TagLibrary;
import org.xdoclet.predicate.HasTag;

import java.util.*;
import java.io.FileInputStream;
import java.io.IOException;

/**
 * plugin producing hibernate config file
 *
 * @author Inger
 * @author Konstantin Pribluda
 * @version $Revision: 1.2 $
 */
public class HibernateConfigPlugin  extends Plugin
{


    private Collection allClasses;
    private String mappingExtension;
    private Properties props;
    private String jdbcdriver;
    private String jdbcurl;
    private String jdbcusername;
    private String jdbcpassword;
    private String jdbcpoolsize;
    private String jndidatasource;
    private String jndiurl;
    private String jndiclass;
    private String dialect;
    private String defaultschema;
    private String jndisessionfactoryname;
    private String userouterjoin;
    private String maxfetchdepth;
    private String jdbcfetchsize;
    private String jdbcbatchsize;
    private String jdbcbatchversioneddata;
    private String jdbcusescrollableresultset;
    private String jdbcusestreamsforbinary;
    private String jdbcusegetgeneratedkeys;
    private String usereflectionoptimizer;
    private String jdbcisolation;
    private String connectionprovider;
    private String cacheprovider;
    private String cacheuseminimalputs;
    private String cacheusequerycache;
    private String cachequerycachefactory;
    private String cacheregionprefix;
    private String transactionfactory;
    private String jtausertransaction;
    private String transactionmanagerlookup;
    private String querysubstitutions;
    private String showsql;
    private String hbm2ddlauto;

    public HibernateConfigPlugin(JellyTemplateEngine jellyTemplateEngine, QDoxMetadataProvider metadataProvider,
                                 WriterMapper writerMapper)
    {
        super(jellyTemplateEngine, metadataProvider, writerMapper);
        setFilereplace("hibernate.cfg.xml");
        setMultioutput(false);
        setMappingextension(".hbm.xml");

        new TagLibrary(metadataProvider);
        allClasses = metadataProvider.getMetadata();
        this.props = new Properties();
    }


    public Collection getMappedClasses()
    {
        return CollectionUtils.select(allClasses,
              new HasTag("hibernate.class", null, null, false));
    }

    public String getMappingPath(JavaClass clazz)
    {
        return clazz.getFullyQualifiedName().replace('.', '/') + mappingExtension;
    }
 
    public void setMappingextension(String mappingExtension)
    {
        this.mappingExtension = mappingExtension;
    }

    /**
     * file name of properties file containing hibernate properties
     */
    public void setProperties(String properties)
    {
        FileInputStream fis = null;
        try
        {
            fis = new FileInputStream(properties);
            props.load(fis);
        }
        catch (IOException e)
        {
            throw new RuntimeException(e);
        }
        finally
        {
            try
            {
                if (fis != null)
                    fis.close();
            }
            catch (IOException e)
            {

            }
        }
    }


    public Map getPropertyMap()
    {
        return props;
    }
    public String getJdbcdriver()
    {
        return jdbcdriver;
    }
    /**
     *  hibernate.connection.driver_class
     */
    public void setJdbcdriver(String jdbcdriver)
    {
        this.jdbcdriver = jdbcdriver;
    }

    public String getJdbcurl()
    {
        return jdbcurl;
    }
    /**
     *  hibernate.connection.url
     */ 
    public void setJdbcurl(String jdbcurl)
    {
        this.jdbcurl = jdbcurl;
    }

    public String getJdbcusername()
    {
        return jdbcusername;
    }
    /**
     * hibernate.connection.username
     */ 
    public void setJdbcusername(String jdbcusername)
    {
        this.jdbcusername = jdbcusername;
    }

    public String getJdbcpassword()
    {
        return jdbcpassword;
    }
    /**
     * hibernate.connection.password
     */
    public void setJdbcpassword(String jdbcpassword)
    {
        this.jdbcpassword = jdbcpassword;
    }

    public String getJdbcpoolsize()
    {
        return jdbcpoolsize;
    }
    /**
    * hibernate.connection.pool_size
    */
    public void setJdbcpoolsize(String jdbcpoolsize)
    {
        this.jdbcpoolsize = jdbcpoolsize;
    }

    public String getJndidatasource()
    {
        return jndidatasource;
    }
    /**
     *  hibernate.connection.datasource   
     */
    public void setJndidatasource(String jndidatasource)
    {
        this.jndidatasource = jndidatasource;
    }

    public String getJndiurl()
    {
        return jndiurl;
    }
    
    /**
     * hibernate.jndi.url
     */
    public void setJndiurl(String jndiurl)
    {
        this.jndiurl = jndiurl;
    }
    public String getJndiclass()
    {
        return jndiclass;
    }

    /**
     * hibernate.jndi.class
     */
    public void setJndiclass(String jndiclass)
    {
        this.jndiclass = jndiclass;
    }
    public String getDialect()
    {
        return dialect;
    }
    /**
     * hibernate.dialect
     */
    public void setDialect(String dialect)
    {
        this.dialect = dialect;
    }

    public String getDefaultschema()
    {
        return defaultschema;
    }
    /**
     * hibernate.default_schema
     */
    public void setDefaultschema(String defaultschema)
    {
        this.defaultschema = defaultschema;
    }

    public String getJndisessionfactoryname()
    {
        return jndisessionfactoryname;
    }
    /**
     * hibernate.session_factory_name
     */
    public void setJndisessionfactoryname(String jndisessionfactoryname)
    {
        this.jndisessionfactoryname = jndisessionfactoryname;
    }

    public String getUserouterjoin()
    {
        return userouterjoin;
    }

    /**
     * hibernate.use_outer_join
     */
    public void setUserouterjoin(String userouterjoin)
    {
        this.userouterjoin = userouterjoin;
    }

    public String getMaxfetchdepth()
    {
        return maxfetchdepth;
    }
    /**
     * hibernate.max_fetch_depth
     */
    public void setMaxfetchdepth(String maxfetchdepth)
    {
        this.maxfetchdepth = maxfetchdepth;
    }

    public String getJdbcfetchsize()
    {
        return jdbcfetchsize;
    }

    /**
     * hibernate.jdbc.fetch_size
     */
    public void setJdbcfetchsize(String jdbcfetchsize)
    {
        this.jdbcfetchsize = jdbcfetchsize;
    }

    public String getJdbcbatchsize()
    {
        return jdbcbatchsize;
    }
    /**
     * hibernate.jdbc.batch_size
     */
    public void setJdbcbatchsize(String jdbcbatchsize)
    {
        this.jdbcbatchsize = jdbcbatchsize;
    }
    
    public String getJdbcbatchversioneddata()
    {
        return jdbcbatchversioneddata;
    }

    /**
     * hibernate.jdbc.batch_versioned_data
     */
    public void setJdbcbatchversioneddata(String jdbcbatchversioneddata)
    {
        this.jdbcbatchversioneddata = jdbcbatchversioneddata;
    }

    public String getJdbcusescrollableresultset()
    {
        return jdbcusescrollableresultset;
    }

    /**
     * hibernate.jdbc.use_scrollable_resultset
     */
    public void setJdbcusescrollableresultset(String jdbcusescrollableresultset)
    {
        this.jdbcusescrollableresultset = jdbcusescrollableresultset;
    }

    public String getJdbcusestreamsforbinary()
    {
        return jdbcusestreamsforbinary;
    }

    /**
     * hibernate.jdbc.use_streams_for_binary
     */
    public void setJdbcusestreamsforbinary(String jdbcusestreamsforbinary)
    {
        this.jdbcusestreamsforbinary = jdbcusestreamsforbinary;
    }

    public String getJdbcusegetgeneratedkeys()
    {
        return jdbcusegetgeneratedkeys;
    }

    /**
     * hibernate.jdbc.use_get_generated_keys
     */
    public void setJdbcusegetgeneratedkeys(String jdbcusegetgeneratedkeys)
    {
        this.jdbcusegetgeneratedkeys = jdbcusegetgeneratedkeys;
    }

    public String getUsereflectionoptimizer()
    {
        return usereflectionoptimizer;
    }

    /**
     * hibernate.cglib.use_reflection_optimizer
     */
    public void setUsereflectionoptimizer(String usereflectionoptimizer)
    {
        this.usereflectionoptimizer = usereflectionoptimizer;
    }

    public String getJdbcisolation()
    {
        return jdbcisolation;
    }

    /**
     * hibernate.connection.isolation
     */
    public void setJdbcisolation(String jdbcisolation)
    {
        this.jdbcisolation = jdbcisolation;
    }

    public String getConnectionprovider()
    {
        return connectionprovider;
    }

    /**
     * hibernate.connection.provider_class
     */
    public void setConnectionprovider(String connectionprovider)
    {
        this.connectionprovider = connectionprovider;
    }

    public String getCacheprovider()
    {
        return cacheprovider;
    }

    /**
     * hibernate.cache.provider_class
     */
    public void setCacheprovider(String cacheprovider)
    {
        this.cacheprovider = cacheprovider;
    }

    public String getCacheuseminimalputs()
    {
        return cacheuseminimalputs;
    }

    /**
     * hibernate.cache.use_minimal_puts
     */
    public void setCacheuseminimalputs(String cacheuseminimalputs)
    {
        this.cacheuseminimalputs = cacheuseminimalputs;
    }

    /**
     * hibernate.cache.use_query_cache
     */
    public String getCacheusequerycache()
    {
        return cacheusequerycache;
    }

    /**
     * hibernate.cache.use_query_cache
     */
    public void setCacheusequerycache(String cacheusequerycache)
    {
        this.cacheusequerycache = cacheusequerycache;
    }

    public String getCachequerycachefactory()
    {
        return cachequerycachefactory;
    }

    /**
     * hibernate.cache.query_cache_factory
     */
    public void setCachequerycachefactory(String cachequerycachefactory)
    {
        this.cachequerycachefactory = cachequerycachefactory;
    }

    public String getCacheregionprefix()
    {
        return cacheregionprefix;
    }

    /**
     * hibernate.cache.region_prefix
     */
    public void setCacheregionprefix(String cacheregionprefix)
    {
        this.cacheregionprefix = cacheregionprefix;
    }

    public String getTransactionfactory()
    {
        return transactionfactory;
    }

    /**
     * hibernate.transaction.factory_class
     */
    public void setTransactionfactory(String transactionfactory)
    {
        this.transactionfactory = transactionfactory;
    }

    public String getJtausertransaction()
    {
        return jtausertransaction;
    }

    /**
     * jta.UserTransaction
     */
    public void setJtausertransaction(String jtausertransaction)
    {
        this.jtausertransaction = jtausertransaction;
    }

    public String getTransactionmanagerlookup()
    {
        return transactionmanagerlookup;
    }

    /**
     * hibernate.transaction.manager_lookup_class
     */
    public void setTransactionmanagerlookup(String transactionmanagerlookup)
    {
        this.transactionmanagerlookup = transactionmanagerlookup;
    }

    public String getQuerysubstitutions()
    {
        return querysubstitutions;
    }

    /**
     * hibernate.query.substitutions
     */
    public void setQuerysubstitutions(String querysubstitutions)
    {
        this.querysubstitutions = querysubstitutions;
    }

    public String getShowsql()
    {
        return showsql;
    }

    /**
     * hibernate.show_sql
     */
    public void setShowsql(String showsql)
    {
        this.showsql = showsql;
    }

    public String getHbm2ddlauto()
    {
        return hbm2ddlauto;
    }

    /**
     * hibernate.hbm2ddl.auto
     */
    public void setHbm2ddlauto(String hbm2ddlauto)
    {
        this.hbm2ddlauto = hbm2ddlauto;
    }

}
