/*
 * Copyright (c) 2003
 * XDoclet Team
 * All rights reserved.
 */
package org.xdoclet.plugin.hibernate;

import com.thoughtworks.qdox.model.AbstractJavaEntity;

/**
 * bean representing hibernate property. it could be property getter, as well as 
 * just a field ( regardless private or public. hibernate does not care )
 */
public class HibernateProperty {
	
	String name;
	String access;
	AbstractJavaEntity entity;
	/**
	* name of hibernate property
	*/
	public String getName() {
		return name;
	}
	
	public void setName(String name) {
		this.name = name;
	}
	
	/** 
	 * access specification. shall be field for fields
	 */
	public String getAccess() {
		return access;
	}
	
	public void setAccess(String access) {
		this.access = access;
	}
	
	public AbstractJavaEntity getEntity() {
		return entity;
	}
	
	
	public void setEntity(AbstractJavaEntity entity) {
		this.entity = entity;
	}
	
	/**
	* puprose of this equality method is to allow proprety overriding
	* properties are equal, if they have the same name, regardless of access.
	* Since we build property map from top to bottom of class hierarchy, we will reflect
	* java behaviour 
	*/
	public boolean equals(Object o) {
		if(o == null) {
			return false;
		}
		
		if(!(o instanceof HibernateProperty)){
			return false;
		}
		
		if(getName() != null) {
			return getName().equals(((HibernateProperty)o).getName());
		}		
		
		return false;
	}
	
	public int hashCode() {
		if(getName() != null) {
			return getName().hashCode();
		}
		return super.hashCode();
	}
	
    public String toString() {
        return "[" + this.name + "][" + this.access + "]";
    }
}
