package org.xdoclet.plugin.jmx.qtags;

/**
 * Defines a jmx managed attribute.  This method will be included in a
 * standard mbean interface and an attribute based on its name will be
 * included as a managed atribute in xmbean metadata. This must be included
 * on any getter or setter you want exposed in a standard mbean. For xmbeans,
 * the methods with these tags will be identified as the getMethod and setMethod
 * for the attribute.
 */
public interface JmxManagedAttributeTag extends com.thoughtworks.qdox.model.DocletTag {
    /**
     * Specifies the access to the managed-attribute: read-only, write-only or read-write.
     *
     * @qtags.allowed-value read-only
     * @qtags.allowed-value write-only
     * @qtags.allowed-value read-write
     */
    String getAccess();

    /**
     * Specifies a description for the managed attribute that can be used in mbean infos.
     *
     */
    String getDescription();

    /**
     * Specifies the persistence policy for this mbean
     *
     * @qtags.allowed-value OnUpdate
     * @qtags.allowed-value NoMoreOftenThan
     * @qtags.allowed-value Never
     * @qtags.allowed-value OnTimer
     */
    String getPersistPolicy();

    /**
     * Specifies when the mbean state will be saved.
     *
     */
    String getPersistPeriod();

    /**
     * Specifies the location for the mbean's state to be saved.
     *
     */
    String getPersistLocation();

    /**
     * Specifies the name to identify the mbeans saved state with.
     *
     */
    String getPersistName();

    /**
     * Specifies how long the mbeans state may be considered valid.
     *
     */
    String getCurrencyTimeLimit();

    /**
     * Specifies what the jboss state management should do when an attribute is updated
     * on a started mbean. (jboss specific).
     *
     */
    String getStateActionOnUpdate();

    /**
     * Specifies the (initial) value for the managed attribute. This is
     * used in the jboss *-service.xml file and the xmbean descriptor.
     * In both cases the result is the value is set as the initial value
     * of the newly deployed mbean.
     *
     */
    String getValue_();

    /**
     * Specifies an additional custom descriptor for the mbean that can be used in mbean infos.
     * This should be of the form name="name" value="value".  To make this work as an xdoclet
     * parameter, you need to escape the double quotes: descriptor="name=\"name\" value=\"value\""
     *
     */
    String getDescriptor();

}
