package org.xdoclet.plugin.jmx.qtags;

/**
 * The jmx.mbean tag identifies the class as an mbean.  It can be used to
 * generate the interface for a standard mbean and the jboss specific
 * xmbean xml descriptor and service.xml file format.
 */
public interface JmxMbeanTag extends com.thoughtworks.qdox.model.DocletTag {
    /**
     * Specifies the superclass for the standard mbean interface.
     *
     */
    String getExtends();

    /**
     * Specifies the object name for mlet or it can be used as a default in an extension template.
     *
     */
    String getName_();

    /**
     * Specifies a description for the mbean that can be used in mbean infos.
     *
     */
    String getDescription();

    /**
     * Specifies the persistence policy for this mbean
     *
     * @qtags.allowed-value OnUpdate
     * @qtags.allowed-value NoMoreOftenThan
     * @qtags.allowed-value Never
     * @qtags.allowed-value OnTimer
     */
    String getPersistPolicy();

    /**
     * Specifies when the mbean state will be saved.
     *
     */
    String getPersistPeriod();

    /**
     * Specifies the location for the mbean's state to be saved.
     *
     */
    String getPersistLocation();

    /**
     * Specifies the name to identify the mbeans saved state with.
     *
     */
    String getPersistName();

    /**
     * Specifies the fully qualified class name of a persistence manager for this mbean (JBoss specific).
     *
     */
    String getPersistenceManager();

    /**
     * Specifies how long the mbeans state may be considered valid.
     *
     */
    String getCurrencyTimeLimit();

    /**
     * Specifies what the jboss state management should do when an attribute is updated
     * on a started mbean. (jboss specific).
     *
     */
    String getStateActionOnUpdate();

    /**
     * Specifies an additional custom descriptor for the mbean that can be used in mbean infos.
     * To make this work as an xdoclet parameter, you need to escape the double quotes:
     * descriptor="name=\"name\" value=\"value\""
     *
     */
    String getDescriptor();

}
