package org.xdoclet.plugin.plugin;

import com.thoughtworks.qdox.model.DocletTag;
import com.thoughtworks.qdox.model.JavaClass;
import com.thoughtworks.qdox.model.JavaMethod;
import org.generama.Plugin;
import org.generama.VelocityTemplateEngine;
import org.generama.WriterMapper;
import org.xdoclet.QDoxMetadataProvider;
import org.xdoclet.plugin.plugin.qtags.TagLibrary;

import java.io.File;
import java.io.IOException;

/**
 * Plugin Plugin is responsible for generation plugin documenation of other plugins.
 *
 * @author Aslak Helles&oslash;y
 * @version $Revision: 1.10 $
 */
public class PluginPlugin extends Plugin {
    private String basedir;

    public PluginPlugin(VelocityTemplateEngine velocityTemplateEngine, QDoxMetadataProvider metadataProvider, WriterMapper writerMapper) {
        super(velocityTemplateEngine, metadataProvider, writerMapper);
        setPackageregex(".");
        setMultioutput(true);
        new TagLibrary(metadataProvider);
    }

    public String getDestinationFilename(Object metadata) {
        try {
            String location = ((JavaClass) metadata).getParentSource().getFile().getCanonicalPath();
            String basename = getTopLocation(location);
            String originalFilename = getMetadataProvider().getOriginalFileName(metadata);
            String name = originalFilename.substring(0, originalFilename.lastIndexOf('.'));
            return basename + "-" + name + ".confluence";
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    public boolean shouldGenerate(Object metadata) {
        JavaClass javaClass = (JavaClass) metadata;
        return javaClass.isA(Plugin.class.getName());
    }

    public String getAllowedValues(JavaMethod method) {
        DocletTag tag = method.getTagByName("generama.property", true);
        if (tag == null) return "";
        return tag.getNamedParameter("allowed-values");
    }

    public String getDefaultValue(JavaMethod method) {
        DocletTag tag = method.getTagByName("generama.property", true);
        if (tag == null) return "";
        return tag.getNamedParameter("default");
    }

    public String isRequired(JavaMethod method) {
        DocletTag tag = method.getTagByName("generama.property", true);
        if (tag == null) return "false";
        if (tag.getNamedParameter("required") == null) return "false";
        return tag.getNamedParameter("required");
    }

    public String getDescription(JavaMethod method) {
        String comment = method.getComment();
        if (comment == null) return "";
        return comment;
    }

    /**
     * Sets the cheese.
     * 
     * @generama.property required="true"
     * default="brie"
     * allowed-values="brie, stilton"
     */
    public void setCheese(String s) {
    }

    /**
     * @throws RuntimeException when a path can not be resolved (@see File.getCanonicalPath)
     */
    public void setBasedir(String basedir) {
        try {
            this.basedir = new File(basedir).getCanonicalPath();
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    public String getBasedir() {
        return basedir;
    }

    /**
     * Returns the first path element from location, after basedir.
     * Used to group Plugins together by their src directories.
     */
    String getTopLocation(String location) {
        if (basedir == null) {
            throw new IllegalStateException("basedir must be set");
        }
        if (!location.startsWith(basedir)) {
            throw new IllegalArgumentException("Basedir must be on top of all scanner dirs(basedir=" + basedir + ", location=" + location + ").");
        }
        int startId = basedir.endsWith(File.separator) ? basedir.length() + 2 : basedir.length() + 1;
        int endIdx = location.indexOf(File.separatorChar, startId);
        return location.substring(startId, endIdx);
    }
}
