package org.xdoclet.plugin.qtags;

import com.thoughtworks.qdox.model.DocletTag;
import com.thoughtworks.qdox.model.JavaClass;
import com.thoughtworks.qdox.model.JavaMethod;

import org.xdoclet.QDoxMetadataProvider;


import java.lang.NoSuchMethodException;
import java.lang.reflect.InvocationTargetException;

/**
 * @author Aslak Helles&oslash;y
 * @author Konstantin Pribluda
 * @version $Revision: 1.8 $
 */
public class QTagUtils {
    /**
     * whether generation is desired for this class.
     * we generate only if class ( interface ) is it is not ignored
     */
    public boolean shouldGenerate(Object metadata) {
        JavaClass javaClass = (JavaClass) metadata;
        boolean isTagClass = javaClass.isA(DocletTag.class.getName());

        boolean enabled = javaClass.getTagByName("qtags.ignore") == null;

        return javaClass.isInterface() && isTagClass && enabled;
    }

    public String getDocletTagName(JavaClass clazz) {
        String name = clazz.getName();

        if (name.endsWith("DocletTag")) {
            name = name.substring(0, name.length() - "DocletTag".length());
        } else if (name.endsWith("Tag")) {
            name = name.substring(0, name.length() - "Tag".length());
        }

        return hyphenate(name, true);
    }

    public String getDocletTagParameterName(JavaMethod method) {
        String result = hyphenate(method.getPropertyName(), false);

        if (method.getName().equals("getName")
                || method.getName().equals("getValue")) {
            throw new RuntimeException(
                    "In order to declare a tag parameter named '" + result
                    + "', declare a method called " + method.getName()
                    + "_() instead of " + method.getName()
                    + "(). This is to avoid clash with the method "
                    + method.getName() + "() in " + DocletTag.class.getName());
        }

        return result;
    }

    public String isParameterRequired(JavaMethod method) {
        if (method.getTagByName("qtags.required") == null) {
            return "false";
        } else {
            return "true";
        }
    }

    private String hyphenate(String camelCase, boolean firstIsDot) {
        StringBuffer sb = new StringBuffer();

        sb.append(Character.toLowerCase(camelCase.charAt(0)));

        char[] chars = camelCase.toCharArray();
        char separator = firstIsDot ? '.' : '-';

        for (int i = 1; i < chars.length; i++) {
            char currentChar = chars[i];

            if (Character.isUpperCase(currentChar)) {
                sb.append(separator).append(Character.toLowerCase(currentChar));
                separator = '-';
            } else {
                sb.append(currentChar);
            }
        }

        String hyphenated = sb.toString();

        if (hyphenated.endsWith("_")) {
            hyphenated = hyphenated.substring(0, hyphenated.length() - 1);
        }

        return hyphenated;
    }
	
	/**
	* register own tag library
	*/
	public void registerLibrary(QDoxMetadataProvider metadataProvider) {
		// try to register our tag library, not for us, but for users...
		// if available
		try {
			Class clazz = getClass().getClassLoader().loadClass("org.xdoclet.plugin.qtags.qtags.TagLibrary");
			if(clazz != null) {
				clazz.getDeclaredConstructor(new Class[] {QDoxMetadataProvider.class}).newInstance(new Object[] {metadataProvider});
			}
		} catch(ClassNotFoundException cnfe) {
			// we just do not care - shit can happen...
		} catch(InstantiationException ie) {
			// we just do not care - shit can happen...
		} catch(IllegalAccessException ie) {
			// we just do not care - shit can happen...
		} catch(NoSuchMethodException ie) {
			// we just do not care - shit can happen...
		} catch(InvocationTargetException ie) {
			// we just do not care - shit can happen...
		}

	}
    
    public boolean allowedOn( JavaClass clazz, String location )  {
        return ! disallowedOn(clazz, location);
    }
    
    public boolean disallowedOn(JavaClass clazz, String location) {
        DocletTag[] tags = clazz.getTagsByName("qtags.location");
        if (tags.length == 0) {
            return false;
        }
        for (int i = 0; i < tags.length; i++) {
            if (location.equals(tags[i].getValue())) {
                return false;
            }
        }
        return true;
    }
}
