package org.xdoclet.plugin.qtags.confluence;

import com.thoughtworks.qdox.model.DocletTag;
import com.thoughtworks.qdox.model.JavaClass;
import com.thoughtworks.qdox.model.JavaMethod;
import com.thoughtworks.qdox.model.Type;
import org.generama.Plugin;
import org.generama.VelocityTemplateEngine;
import org.generama.WriterMapper;
import org.xdoclet.QDoxMetadataProvider;
import org.xdoclet.plugin.qtags.QTagUtils;

import java.util.Collection;
import java.util.LinkedList;

/**
 * Todo: maybe rename this plugin, since it doesn't generate xdoc anymore ;)
 * @author Aslak Helles&oslash;y
 * @version $Revision: 1.3 $
 */
public class QTagConfluencePlugin extends Plugin {
    private String namespace;
    private QTagUtils qTagUtils = new QTagUtils();

    public QTagConfluencePlugin(VelocityTemplateEngine templateEngine, QDoxMetadataProvider metadataProvider,
                          WriterMapper writerMapper) {
        super(templateEngine, metadataProvider, writerMapper);

        setFilereplace("qtags.confluence");
        setMultioutput(false);
		
		qTagUtils.registerLibrary(metadataProvider);
    }

    public boolean shouldGenerate(Object metadata) {
        return qTagUtils.shouldGenerate(metadata);
    }

    public QTagUtils getQTagUtils() {
        return qTagUtils;
    }

    public String getNamespace() {
        return namespace;
    }

    public void setNamespace(String namespace) {
        this.namespace = namespace;
    }

    public Collection getMethodsWithTagParameters(JavaClass javaClass) {
        LinkedList methods = new LinkedList();
        JavaMethod[] allMethods = javaClass.getMethods(true);
        for (int i = 0; i < allMethods.length; i++) {
            if (hasTagParameters(allMethods[i])) {
                methods.add(allMethods[i]);
            }
        }
        return methods;
    }

    
    private boolean hasTagParameters(JavaMethod method) {
        String parent=method.getParentClass().getFullyQualifiedName();
        return method.isPropertyAccessor() &&
            !(parent.equals("java.lang.Object")) &&
            !(parent.equals("com.thoughtworks.qdox.model.DocletTag"));

    }

    public String getAllowedValues(JavaMethod method) {
        if (method.getReturns().isA(new Type("boolean")) || method.getReturns().isA(new Type("java.lang.Boolean"))) {
            return "true, false";
        } else if (method.getTagByName("qtags.allowed-value") != null ) {
            StringBuffer allowedValues = new StringBuffer();
            DocletTag[] tagsByName = method.getTagsByName("qtags.allowed-value");
            for (int i = 0; i < tagsByName.length; i++) {
                if (i>0) allowedValues.append(", ");
                allowedValues.append(tagsByName[i].getValue());
            }
            return allowedValues.toString();
        } else {
            return "Any value";
        }
    }

    public String getDefaultValue(JavaMethod method) {
        DocletTag tag = method.getTagByName("qtags.parameter");
        if (tag==null) return "";

        return notNull(tag.getNamedParameter("default"));
    }

    public String getDescription(JavaMethod method) {
        return notNull(method.getComment());
    }

    private String notNull(String s) {
        if (s == null) return "";
        return s;
    }
}
