package org.xdoclet.plugin.web;

import com.thoughtworks.qdox.model.JavaClass;

import java.util.Comparator;

/**
 * A Comparator to sort <code>JavaClass</code>es by a given attribute of a given
 * tag. The lower order values will be processed first.
 * Classes with no order attribute will be last.
 *
 * The sorting uses the same contract as the String compareTo method.
 *
 * Example of usage is to force the order in which <code>Filter</code>s are applied
 * to an http request, in the web plugin.
 *
 * <p><br/><br/>created     Sep 22, 2003 2:21:56 AM</p>
 *
 * @see String compareTo(String other) for the contract of ordering
 *
 * @author     <a href="mailto:greg_greg@freestart.hu">Grgory Joseph</a>
 * @author     $Author: rinkrank $ (last edit)
 * @version    $Revision: 1.1 $
 */
public class ClassAttributeComparator implements Comparator {
    private String tag;
    private String att;

    public ClassAttributeComparator(String tag, String attribute) {
        this.tag = tag;
        this.att = attribute;
    }

    public int compare(Object o1, Object o2) {
        JavaClass clazz1 = (JavaClass) o1;
        JavaClass clazz2 = (JavaClass) o2;

        String order1 = clazz1.getNamedParameter(tag, att);
        String order2 = clazz2.getNamedParameter(tag, att);

        if (order1 == null && order2 == null) {
            return 0;
        } else if (order1 == null) {
            return 1;
        } else if (order2 == null) {
            return -1;
        } else {
            return order1.compareTo(order2);
        }
    }
}
