package org.xdoclet.plugin.web;

import com.thoughtworks.qdox.model.DocletTag;
import com.thoughtworks.qdox.model.JavaClass;
import com.thoughtworks.qdox.model.JavaMethod;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.collections.Predicate;
import org.generama.JellyTemplateEngine;
import org.generama.WriterMapper;
import org.generama.Plugin;
import org.xdoclet.QDoxMetadataProvider;
import org.xdoclet.plugin.web.qtags.TagLibrary;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;

/**
 * The plugin to generate tld files for jsp taglibs.
 *
 * TODO : inspect the difference between variables declared in the tld and a simple tei class
 * TODO : inspect what uses tei classes can have other than the basic one (getVariableInfo(return new VariableInfo[] {...}))
 * TODO : eventually make a plugin to generate simple tei classes, if it has any difference to declaring variables in the tag
 *
 *
 * <p><br/><br/>created     Oct 20, 2003 10:04:09 PM</p>
 * @author     <a href="mailto:greg_greg@freestart.hu">Gr&eacute;gory Joseph</a>
 * @author     $Author: rinkrank $ (last edit)
 * @version    $Revision: 1.11 $
 */
public class TaglibPlugin extends Plugin {
    public static final String PUBLIC_ID_1_2 = "-//Sun Microsystems, Inc.//DTD JSP Tag Library 1.2//EN";
    public static final String SYSTEM_ID_1_2 = "http://java.sun.com/dtd/web-jsptaglibrary_1_2.dtd";

    private static final String DEFAULT_TAGLIBVERSION = "1.0";
    private static final String DEFAULT_JSPVERSION = "1.2";
    private static final String DEFAULT_SHORTNAME = "taglib";
    /** maps jsp version to servlet version */
    private static final HashMap servletVersions = new HashMap();

    static {
        servletVersions.put("1.1", "2.2");
        servletVersions.put("1.2", "2.3");
        servletVersions.put("2.0", "2.4");
    }

    private String publicId = PUBLIC_ID_1_2;
    private String systemId = SYSTEM_ID_1_2;

    private Collection allClasses;
    private WebUtils webUtils;
    // mandatory plugin attributes
    private String jspversion;
    private String taglibversion;
    private String shortname;
    // optional plugin attributes
    private String uri;
    private String displayname;
    private String smallicon;
    private String largeicon;
    private String description;
    private String validator; // a class
    private boolean includeListeners = true;

    public TaglibPlugin(JellyTemplateEngine jellyTemplateEngine, QDoxMetadataProvider metadataProvider, WriterMapper writerMapper) {
        super(jellyTemplateEngine, metadataProvider, writerMapper);

        setMultioutput(false);
        setFilereplace("taglib.tld");
        setTaglibversion(DEFAULT_TAGLIBVERSION);
        setJspversion(DEFAULT_JSPVERSION);
        setShortname(DEFAULT_SHORTNAME);

        new TagLibrary(metadataProvider);

        allClasses = metadataProvider.getMetadata();
    }

    /**
     * Returns a Collection of classes implementing javax.servlet.jsp.tagext.Tag
     * and which have a jsp.tag tag.
     *
     * @return
     */
    public Collection getTags() {
        // filter out non-tags classes
        ArrayList tags = new ArrayList();
        Predicate predicate = new ClassTagPredicate("javax.servlet.jsp.tagext.Tag", "jsp.tag");
        CollectionUtils.select(allClasses, predicate, tags);
        return tags;
    }

    /**
     * Returns a collection of JavaMethod from a class, which have a
     * jsp.attribute tag.
     *
     * @param clazz
     * @return
     */
    public Collection getAttributeMethods(JavaClass clazz) {
        Collection attributeMethods = new ArrayList();
        JavaMethod[] meths = clazz.getMethods();
        for (int i = 0; i < meths.length; i++) {
            DocletTag tag = meths[i].getTagByName("jsp.attribute");
            if (tag != null) {
                attributeMethods.add(meths[i]);
            }
        }
        return attributeMethods;
    }

    public WebUtils getWebUtils() throws ClassNotFoundException {
        String servletVersion = (String) servletVersions.get(jspversion);
        if (webUtils == null) {
            webUtils = new WebUtils(servletVersion);
        }
        return webUtils;
    }

    //== plugin attributes ==
    public String getIncludelisteners() {
        return includeListeners ? "true" : "false";
    }

    /**
     * Wheter or not to include the listeners definitions in the generated tld file.
     * Is true if not specified.
     * @param includeListenersStr "true" or "false"
     */
    public void setIncludelisteners(String includeListenersStr) {
        this.includeListeners = includeListenersStr.equalsIgnoreCase("true");
    }

    public String getJspversion() {
        return jspversion;
    }

    public void setJspversion(String jspversion) {
        this.jspversion = jspversion;
    }

    public String getTaglibversion() {
        return taglibversion;
    }

    public void setTaglibversion(String taglibversion) {
        this.taglibversion = taglibversion;
    }

    public String getShortname() {
        return shortname;
    }

    public void setShortname(String shortname) {
        this.shortname = shortname;
    }

    public String getUri() {
        return uri;
    }

    public void setUri(String uri) {
        this.uri = uri;
    }

    public String getDisplayname() {
        return displayname;
    }

    public void setDisplayname(String displayname) {
        this.displayname = displayname;
    }

    public String getSmallicon() {
        return smallicon;
    }

    public void setSmallicon(String smallicon) {
        this.smallicon = smallicon;
    }

    public String getLargeicon() {
        return largeicon;
    }

    public void setLargeicon(String largeicon) {
        this.largeicon = largeicon;
    }

    public String getDescription() {
        return description;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    public String getValidator() {
        return validator;
    }

    public void setValidator(String validator) {
        this.validator = validator;
    }

    public String getPublicId() {
        return publicId;
    }

    public String getSystemId() {
        return systemId;
    }
}
