package org.xdoclet.plugin.web;

import com.thoughtworks.qdox.model.DocletTag;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.collections.Predicate;
import org.generama.JellyTemplateEngine;
import org.generama.WriterMapper;
import org.generama.Plugin;
import org.xdoclet.QDoxMetadataProvider;
import org.xdoclet.plugin.web.qtags.TagLibrary;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

/**
 *
 * TODO : check qtags, xtags
 *
 * TODO: add getMetadataProvider() to AbstractPlugin instead of keeping the allClass reference ?
 *
 * TODO : validate generated XML ?
 *
 * TODO : mandatory tags/attributes? (filter-name, filter-class, filter-mapping, ...)
 *
 * TODO : filter-mapping: check that a mapping has only of one url-pattern or servlet-name
 * TODO : filter-mapping: check that a servlet-name mapping points to an existing servlet? i wouldn't do it, because
 *                        a servlet might be declared in a merge file.
 *
 * @author     <a href="mailto:greg_greg@freestart.hu">Grgory Joseph</a>
 * @author     Aslak Helles&oslash;y
 * @version    $Revision: 1.14 $
 */
public class WebPlugin extends Plugin {
    public static final String PUBLIC_ID_2_2 = "-//Sun Microsystems, Inc.//DTD Web Application 2.2//EN";
    public static final String SYSTEM_ID_2_2 = "http://java.sun.com/dtd/web-app_2_2.dtd";
    public static final String PUBLIC_ID_2_3 = "-//Sun Microsystems, Inc.//DTD Web Application 2.3//EN";
    public static final String SYSTEM_ID_2_3 = "http://java.sun.com/dtd/web-app_2_3.dtd";

    private String publicId = PUBLIC_ID_2_3;
    private String systemId = SYSTEM_ID_2_3;

    private static Map filterInterface = new HashMap();

    static {
        filterInterface.put(WebUtils.SERVLET_23, new String[]{"javax.servlet.Filter"});
        filterInterface.put(WebUtils.SERVLET_24, new String[]{"javax.servlet.Filter"});
    }

    private static String servletInterface = "javax.servlet.Servlet";
    private static String[] allWebInterfaces = {"javax.servlet.Filter", servletInterface,
                                                "javax.servlet.ServletContextAttributeListener", "javax.servlet.ServletContextListener",
                                                "javax.servlet.ServletRequestAttributeListener", "javax.servlet.ServletRequestListener",
                                                "javax.servlet.http.HttpSessionActivationListener", "javax.servlet.http.HttpSessionAttributeListener",
                                                "javax.servlet.http.HttpSessionBindingListener", "javax.servlet.http.HttpSessionListener"}; // TODO avoid repeating listenerInterfaces

    private Collection allClasses;
    private WebUtils webUtils;
    private String servletVersion = WebUtils.SERVLET_23;

    public WebPlugin(JellyTemplateEngine jellyTemplateEngine, QDoxMetadataProvider metadataProvider, WriterMapper writerMapper) {
        super(jellyTemplateEngine, metadataProvider, writerMapper);

        setMultioutput(false);
        setFilereplace("web.xml");

        new TagLibrary(metadataProvider);
        allClasses = metadataProvider.getMetadata();
    }

    //== plugin attributes ==
    public String getServletVersion() {
        return servletVersion;
    }

    public void setServletVersion(String servletVersion) {
        this.servletVersion = servletVersion;
    }

    //== util methods used by jelly script ==
    public WebUtils getWebUtils() throws ClassNotFoundException {
        if (webUtils == null) {
            webUtils = new WebUtils(servletVersion);
        }
        return webUtils;
    }

    /**
     * Returns an ordered collection of all the filters passed to the plugin.
     *
     * TODO: document the "order" attribute.
     * TODO: more test cases with order? maybe not xmltestcases then, but smaller units..
     * TODO: use "before"&"after" attributes instead of "order" ?
     *
     * TODO : return empty collection if servlet-api-version < 2.3
     *
     * @return
     * @see ClassTagPredicate used to filter out non-filters
     * @see ClassAttributeComparator used to sort filters
     */
    public Collection getFilters() {
        // filter out non-filter classes
        ArrayList filters = new ArrayList();
        String[] interfaces = (String[]) filterInterface.get(servletVersion);
        if (interfaces != null) {
            Predicate predicate = new ClassTagPredicate(interfaces, "web.filter");
            CollectionUtils.select(allClasses, predicate, filters);

            // sort filters by the order attribute of the @web.filter tag
            Collections.sort(filters, new ClassAttributeComparator("web.filter", "order"));
        }
        return filters;
    }

    public Collection getServlets() {
        ArrayList servlets = new ArrayList();
        Predicate predicate = new ClassTagPredicate(servletInterface, "web.servlet");
        CollectionUtils.select(allClasses, predicate, servlets);

        // sort servlets by the order attribute of the @web.servlet tag
        Collections.sort(servlets, new ClassAttributeComparator("web.servlet", "order"));

        return servlets;
    }

    /**
     * TODO : cache the result of this? (maybe it's one of the reasons the plugin is slow??)
     * @return
     */
    public Collection getAllWebClasses() {
        ArrayList all = new ArrayList();
        Predicate predicate = new ClassTagPredicate(allWebInterfaces);
        CollectionUtils.select(allClasses, predicate, all);

        // TODO: any order for this? maybe by classname.

        return allClasses;
    }

    public boolean needsServletRunAs(DocletTag tag) {
        return (tag.getNamedParameter("run-as") != null)
            && (servletVersion.compareTo(WebUtils.SERVLET_23)>=0);
    }

    public String getPublicId() {
        return publicId;
    }

    public String getSystemId() {
        return systemId;
    }
}
