package org.xdoclet.plugin.web;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.collections.Predicate;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;

/**
 * Common methods used by web related plugins.
 *
 *
 * <p><br/><br/>created     Oct 26, 2003 2:15:36 AM</p>
 * @author     <a href="mailto:greg_greg@freestart.hu">Gr&eacute;gory Joseph</a>
 * @author     $Author: moinsmoins $ (last edit)
 * @version    $Revision: 1.6 $
 */
public class WebUtils {
    protected static final String SERVLET_22 = "2.2";
    protected static final String SERVLET_23 = "2.3";
    protected static final String SERVLET_24 = "2.4";

    private String servletVersion;

    public WebUtils(String servletVersion) throws ClassNotFoundException {
        // Try to load one of the javax.servlet classes so the user can
        // get an early warning if she has forgotten to stick it on the classpath
        try {
            getClass().getClassLoader().loadClass("javax.servlet.Servlet");
        } catch (ClassNotFoundException ex) {
            throw new ClassNotFoundException("The servlet api jar must be in the classpath (" + ex.getMessage() + ")");
        }
        this.servletVersion = servletVersion;
    }

    private static final HashMap listenerInterfaces = new HashMap();
    static {
        listenerInterfaces.put(SERVLET_24, new String[]{
            "javax.servlet.ServletContextAttributeListener", "javax.servlet.ServletContextListener",
            "javax.servlet.ServletRequestAttributeListener", "javax.servlet.ServletRequestListener",
            "javax.servlet.http.HttpSessionActivationListener", "javax.servlet.http.HttpSessionAttributeListener",
            "javax.servlet.http.HttpSessionBindingListener", "javax.servlet.http.HttpSessionListener"});
        listenerInterfaces.put(SERVLET_23, new String[]{
            "javax.servlet.ServletContextAttributeListener", "javax.servlet.ServletContextListener",
            "javax.servlet.http.HttpSessionActivationListener", "javax.servlet.http.HttpSessionAttributeListener",
            "javax.servlet.http.HttpSessionBindingListener", "javax.servlet.http.HttpSessionListener"});
        listenerInterfaces.put(SERVLET_22, new String[]{});
    }

    /**
     * Returns an ordered collection of all the listeners present in the metadata.
     *
     *
     * @param metaData the metaDate from the MetaDataProvider, a Collection of JavaClass.
     * @return
     * @see ClassTagPredicate used to filter out non-listeners
     * @see ClassAttributeComparator used to sort listeners
     */
    public Collection getListeners(Collection metaData) {
        // filter out non-listener classes
        ArrayList listeners = new ArrayList();
        String[] interfaces = (String[]) listenerInterfaces.get(servletVersion);
        if (interfaces != null) {
            Predicate predicate = new ClassTagPredicate(interfaces, "web.listener");
            CollectionUtils.select(metaData, predicate, listeners);

            // sort listener by the order attribute of the @web.listener tag
            Collections.sort(listeners, new ClassAttributeComparator("web.listener", "order"));
        }
        return listeners;
    }

    /**
     * Checks if the currently selected servlet api version is at least the one
     * passed as parameter. Currently does a lexicographic comparison between
     * string version numbers.
     * @see WebUtils SERVLET_* constants for currently supported version numbers.
     * @param minimalVersion
     * @return
     */
    public boolean isMinimumServletVersion(String minimalVersion) {
        return servletVersion.compareTo(minimalVersion)>=0;
    }
}
