package org.xdoclet.plugin.web;

import org.generama.JellyTemplateEngine;
import org.generama.MetadataProvider;
import org.generama.Plugin;
import org.generama.WriterMapper;
import org.generama.tests.AbstractXMLGeneratingPluginTestCase;
import org.xdoclet.QDoxMetadataProvider;

import java.io.File;
import java.io.IOException;
import java.net.URL;

/**
 * @author     <a href="mailto:greg_greg@freestart.hu">Grgory Joseph</a>
 * @author     Aslak Helles&oslash;y
 * @version    $Revision: 1.9 $
 */
public class WebPluginTestCase extends AbstractXMLGeneratingPluginTestCase {
    private String servletVersion;

    protected void setUp() throws Exception {
        super.setUp();
        servletVersion = "2.3"; // make sure it's reset to 2.3 for every test, even if a specific test changes the version number

        registerDtd(WebPlugin.PUBLIC_ID_2_2, getResourceURLRelativeToThisPackage("web-app_2_2.dtd"));
        registerDtd(WebPlugin.PUBLIC_ID_2_3, getResourceURLRelativeToThisPackage("web-app_2_3.dtd"));
    }

    protected Plugin createPlugin(MetadataProvider metadataProvider, WriterMapper writerMapper) throws Exception {
        return new WebPlugin(new JellyTemplateEngine(), (QDoxMetadataProvider) metadataProvider, writerMapper);
    }

    protected MetadataProvider createMetadataProvider() throws IOException {
        String basedir = System.getProperty("xdoclet.web.home");
        assertNotNull(basedir);
        File testAppDir = new File(basedir, "../testapp-web/src/main/java/org/xdoclet/testapp/web");
        return new QDoxMetadataProvider(testAppDir);
    }

    protected URL getExpected() throws IOException {
        if (servletVersion.equals("2.2")) {
            return getResourceRelativeToThisPackage("../../testapp/web/expected/web-22.xml");
        } else {
            return getResourceRelativeToThisPackage("../../testapp/web/expected/web-23.xml");
        }
    }

    public void testWithServlet22() throws Throwable {
        servletVersion = "2.2";
        ((WebPlugin) plugin).setServletVersion(servletVersion);
        testGenerateContent();
    }
}
