package org.xdoclet.plugin.xtags;

import org.generama.MetadataProvider;
import org.dom4j.Document;
import org.dom4j.Element;
import org.dom4j.DocumentException;
import org.dom4j.io.SAXReader;

import java.io.Reader;
import java.io.FileReader;
import java.io.FileNotFoundException;
import java.util.Collection;
import java.util.List;

import com.thoughtworks.qdox.model.DocletTagFactory;

/**
 * @author Aslak Helles&oslash;y
 * @version $Revision: 1.4 $
 */
public class XTagsMetadataProvider implements MetadataProvider {
    private Reader reader;

    public void setXtagsxml(String fileName) throws FileNotFoundException {
        reader = new FileReader(fileName);
    }

    public Collection getMetadata() {
        SAXReader saxReader = new SAXReader();
        try {
            Document document = saxReader.read(reader);
            List result = document.selectNodes("//xdoclet/namespace/tags/tag");
            return result;
        } catch (DocumentException e) {
            throw new RuntimeException("Couldn't read XML", e);
        }
    }

    public String getOriginalFileName(Object metadata) {
        Element tagElement = (Element) metadata;
        String tagName = tagElement.selectSingleNode("name").getText();
        int namespacePos = tagName.indexOf('.');
        if (namespacePos == -1) {
            namespacePos = tagName.indexOf(':');
        }
        String nameSpace = tagName.substring(0, namespacePos);
        String rest = tagName.substring(namespacePos + 1);

        String nameSpaceCapitalised = Character.toUpperCase(nameSpace.charAt(0)) + nameSpace.substring(1);
        String restCamelCase = toCamelCase(rest);
        return nameSpaceCapitalised + restCamelCase + "Tag.java";
    }

    public String getOriginalPackageName(Object metadata) {
        return "";
    }

    public DocletTagFactory getDocletTagFactory() {
        return null;
    }

    public String toCamelCase(String hyphenated) {
        StringBuffer sb = new StringBuffer();
        sb.append(Character.toUpperCase(hyphenated.charAt(0)));
        char[] chars = hyphenated.toCharArray();
        boolean lastWasDash = false;
        for (int i = 1; i < chars.length; i++) {
            char currentChar = chars[i];
            if (currentChar == '-') {
                lastWasDash = true;
            } else {
                if (lastWasDash) {
                    currentChar = Character.toUpperCase(currentChar);
                }
                sb.append(currentChar);
                lastWasDash = false;
            }
        }
        return sb.toString();
    }
}
