package org.xdoclet.plugin.xtags;

import org.dom4j.Element;
import org.generama.VelocityTemplateEngine;
import org.generama.WriterMapper;
import org.generama.defaults.JavaGeneratingPlugin;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.StringReader;
import java.util.List;

/**
 * This plugin converts legacy xtags.xml files into new XDoclet2
 * tag classes. These should be processed by the qtags plugins.
 *
 * @author Aslak Helles&oslash;y
 * @version $Revision: 1.6 $
 */
public class XTagsXMLMigratorPlugin extends JavaGeneratingPlugin {
    public XTagsXMLMigratorPlugin(VelocityTemplateEngine velocityTemplateEngine, XTagsMetadataProvider metadataProvider, WriterMapper writerMapper) {
        super(velocityTemplateEngine, metadataProvider, writerMapper);
        setMultioutput(true);
    }

    public String formatAsJavaDoc(String indent, String text) throws IOException {
        StringBuffer sb = new StringBuffer();
        StringReader sr = new StringReader(text);
        BufferedReader br = new BufferedReader(sr);
        String line = null;
        while ((line = br.readLine()) != null) {
            line = line.trim();
            if (!line.equals("")) {
                sb.append(indent + "* " + line.trim()).append("\n");
            }
        }
        return sb.toString();
    }

    public String getMethodName(Element parameterElement) {
        String parameterName = parameterElement.selectSingleNode("name").getText();
        if("".equals(parameterName)) {
            // no parameter - just the name
            parameterName = "fixme";
        }
        String typeAndPrefix = parameterElement.attributeValue("type").equals("bool") ? "boolean is" : "String get";

        XTagsMetadataProvider mdp = (XTagsMetadataProvider) getMetadataProvider();
        String result = typeAndPrefix + mdp.toCamelCase(parameterName);
        if (result.equals("String getClass") || result.equals("String getName") || result.equals("String getValue")) {
            result += "_";
        }
        return result;
    }

    public List getAllowedValues(Element parameterElement) {
        return parameterElement.selectNodes("option-sets/option-set/options/option");
    }
}
