import java.io.File
import java.net.ServerSocket
import java.util.HashMap
import groovy.net.xmlrpc.XMLRPCServerProxy

class ConfluenceAccess {
    static main(args) {
        if (args.size()!=3) {
          println "usage: ConfluenceAccess userId password rootDir"
          System.exit(-1)
        }
        new ConfluenceAccess(userId:args[0], password:args[1]).upload(args[2])
    }

    // properties
    userId
    password
    serverProxy = new XMLRPCServerProxy("http://docs.codehaus.org/rpc/xmlrpc")
    allPagesMap = [:]
    xdocletPluginPageId = '1921'

    upload(rootDir) {
        // check directories exist
        if (!new File(rootDir).exists() || !new File(rootDir, 'plugin-doc').exists() || !new File(rootDir, 'tag-doc').exists()) {
            println "There must be plugin-doc and tag-doc directories in " + rootDir
            System.exit(-1)
        }

        // get all pages
        serverProxy.confluence1.login(userId, password) { token |
            try {
                allPages = serverProxy.confluence1.getPages(token, 'XDOCLET')
                allPages.each { page |
                    allPagesMap.put(page['title'], page)
                }
            } finally {
                serverProxy.confluence1.logout(token)
            }
        }

        println ""
        println " ... creating group pages if needed ..."
        println ""

        // create group pages if needed
        new File(rootDir, 'plugin-doc').eachFile() { file |
            //name = 'plugin-' + file.name
            //idx = name.length() - 11; // '.confluence'
            //groupName = file.name.substring(0, idx)
            secondDashIdx = file.name.indexOf('-', 7)
            groupName = file.name.substring(0, secondDashIdx)

            if (allPagesMap[groupName] == null) {
                println 'Creating group page: ' + groupName
                uploadPage(groupName, 'Homepage of ' + groupName + '\n{children:all=true}', 'XDocletPlugins')
            }
        }

        println ""
        println " ... uploading plugin doc pages ..."
        println ""

        // upload plugin doc pages
        new File(rootDir, 'plugin-doc').eachFile() { file |
            // plugin-ejb-EjbJarXmlPlugin.confluence
            // EjbJarXmlPlugin
            secondDashIdx = file.name.indexOf('-', 7)
            parentTitle = file.name.substring(0, secondDashIdx)
            pluginName = file.name.substring(parentTitle.length()+1, file.name.length() - 11)
            uploadPage(pluginName, file.getText(), parentTitle)
        }

        println ""
        println " ... uploading tag doc pages ..."
        println ""

        // upload tag doc pages
        new File(rootDir, 'tag-doc').eachFile() { file |
            //ejb.confluence
            //EjbTags
            name = file.name.substring(0, file.name.length() - 11)
            parentTitle = 'plugin-' + name
            title = name[0].toUpperCase() + name[1 .. name.length()-1] + 'Tags'
            uploadPage(title, file.getText(), parentTitle)
        }
    }

    uploadPage(title, content, parentTitle) {
        serverProxy.confluence1.login(userId, password) { token |
            try {
                parentId = xdocletPluginPageId
                parent = allPagesMap.get(parentTitle)
                if (parent != null) {
                    parentId = parent['id']
                }
                origPage = allPagesMap.get(title)
                if (origPage != null) {
                    println title + " already exists... retrieving original..." // for the sake of knowing version...
                    origPage = serverProxy.confluence1.getPage(token, origPage['id'])
                    page = ['space' : 'XDOCLET',
                            'title' : title,
                            'content' : content,
                            'id' : origPage['id'],
                            'parentId' : parentId]
                    if (origPage['version'] == null) {
                        page.version = '1'
                    } else {
                        page.version = origPage['version']
                    }
                } else {
                    page = ['space' : 'XDOCLET',
                            'title' : title,
                            'content' : content,
                            'parentId' : parentId]
                }

                serverProxy.confluence1.storePage(token, page) { result |
                    println "   --> uploaded: " + result['title'] + " - version " + result['version'] + " (in " + parentTitle + ")"
                    allPagesMap.put(result['title'], result)
                }
            } finally {
                serverProxy.confluence1.logout(token)
            }
        }
    }

}
