package org.xdoclet.sampleapp.web;

import org.xdoclet.sampleapp.web.Guestbook;

import javax.servlet.jsp.tagext.TagSupport;
import javax.servlet.jsp.JspException;
import java.io.IOException;
import java.text.DateFormat;
import java.util.Locale;

/**
 * A jsp tag that will expose variables to it's body for displaying the
 * contents of a guestbook object, by looping on the guestbook entries.
 *
 * @jsp.tag name="gbookentry"
 *          display-name="GuestbookEntryTag"
 *          body-content="JSP"
 *          description="Tag exposing one guestbook entry."
 *
 * @jsp.variable name-given="date"
 *               declare="true"
 *               scope="NESTED"
 * @jsp.variable name-given="name"
 *               declare="true"
 *               scope="NESTED"
 * @jsp.variable name-given="message"
 *               declare="true"
 *               scope="NESTED"
 *
 * @author     <a href="mailto:greg_greg@freestart.hu">Gr&eacute;gory Joseph</a>
 * @author     $Author: moinsmoins $ (last edit)
 * @version    $Revision: 1.1 $
 */
public class GuestbookEntryTag extends TagSupport {
    private DateFormat df = DateFormat.getDateTimeInstance(DateFormat.MEDIUM, DateFormat.SHORT, Locale.getDefault());
    private int index;
    private Guestbook.GuestbookEntry[] entries;

    public int doStartTag() throws JspException {
        GuestbookTag gbookTag = (GuestbookTag) findAncestorWithClass(this, GuestbookTag.class);
        if (gbookTag==null) {
            throw new JspException("GuestbookEntryTag must be enclosed in a GuestbookTag.");
        }

        this.entries = gbookTag.getAllMessages();
        this.index = 0;

        if (entries.length>0) {
            exposeCurrentEntry();
            return EVAL_BODY_INCLUDE;
        }
        return SKIP_BODY;
    }

    public int doAfterBody() throws JspException {
        if (index<entries.length) {
            exposeCurrentEntry();
            return EVAL_BODY_AGAIN;
        }
        return SKIP_BODY;
    }

    private void exposeCurrentEntry() {
        pageContext.setAttribute("date", df.format(entries[index].getDate()));
        pageContext.setAttribute("name", entries[index].getName());
        pageContext.setAttribute("message", entries[index].getMessage());

        index++;
    }
}
