package org.xdoclet.testapp.pojo;

import java.util.Collection;

/**
 * Example of a JavaBean with tags to generate the associated BeanInfo class.
 *
 * @javabean.class
 *     name="Simple Bean"
 *     display-name="Very Simple Bean"
 *     customizer="java.beans.Customizer"
 *     short-description="Simple example of JavaBean BeanInfo generation"
 *     expert="true"
 *     hidden="false"
 *     preferred="true"
 *     stop-class="org.xdoclet.plugins.beaninfo.testapp.FullOne"
 *
 * @javabean.icons
 *     color16="/toolbarButtonGraphics/general/Stop16.gif"
 *
 * @javabean.attribute
 *    name="literal"
 *    value="A sample attribute"
 *
 * @javabean.attribute
 *    name="expression"
 *    value="new StringBuffer()"
 *    rtexprvalue="true"
 */
public class MyBean {
    /** An int field. */
    protected int count = 0;

    /** A boolean field. */
    protected boolean flag = false;

    /** A String field. */
    protected String name = "";

    /** A Class field. */
    protected Collection content = null;

    /** Constructs a new SimpleBean. */
    public MyBean() {
    }

    /**
     * The first method
     *
     * @javabean.method
     *     display-name="My First Method"
     *     short-description="Example of method without parameters"
     *
     * @javabean.attribute
     *    name="attribute1"
     *    value="A sample attribute"
     *
     * @javabean.attribute
     *    name="attribute2"
     *    value="new StringBuffer()"
     *    rtexprvalue="true"
     */
    public void myFirstMethod() {
    }

    /**
     * The second method
     *
     * @param param1 Description of the Parameter 1
     * @param param2 Description of the Parameter 2
     *
     * @javabean.method
     *     display-name="My Second Method"
     *     short-description="Example of method with parameters"
     *
     * @javabean.parameter
     *     name="param1"
     *     display-name="Parameter 1"
     *     short-description="Parameter 1 of mySecondMethod"
     *
     * @javabean.parameter
     *     name="param2"
     *     display-name="Parameter 2"
     *     short-description="Parameter 2 of mySecondMethod"
     *
     * @javabean.parameter
     *     name="param3"
     *     display-name="Parameter 3"
     *     short-description="Parameter 3 of mySecondMethod"
     *
     * @javabean.parameter
     *     name="param4"
     *     display-name="Parameter 4"
     *     short-description="Parameter 4 of mySecondMethod"
     */
    public void mySecondMethod(String param1, boolean param2,
                               Collection param3, short param4) {
    }

    /**
     * @javabean.property
     *     display-name="Is a Flag"
     *     bound="true"
     *     hidden="false"
     *     short-description="Is it a flag"
     */
    public boolean isFlag() {
        return flag;
    }

    public void setFlag(boolean flag) {
        this.flag = flag;
    }

    public String getName() {
        return name;
    }

    /**
     * @javabean.property
     *     display-name="Name"
     *     short-description="Name of the entry"
     *     expert="true"
     *     hidden="false"
     *     editor="org.xdoclet.plugins.beaninfo.testapp.FullOne"
     *
     * @javabean.attribute
     *    name="literal"
     *    value="A sample attribute"
     *
     * @javabean.attribute
     *    name="expression"
     *    value="new StringBuffer()"
     *    rtexprvalue="true"
     */
    public void setName(String name) {
        this.name = name;
    }

    /**
     * @javabean.property
     *     display-name="Message(s)"
     *     short-description="Number of messages in Queue"
     *     default="true"
     */
    public int getCount() {
        return count;
    }

    /**
     * @javabean.property
     *     display-name="Full content"
     *     short-description="Full content of the Bean"
     */
    public Collection getContent() {
        return content;
    }
}
