package org.xdoclet.testapp.web;

import javax.servlet.jsp.JspTagException;
import javax.servlet.jsp.JspWriter;
import javax.servlet.jsp.tagext.TagSupport;
import java.io.IOException;
import java.util.Date;

/**
 * Simple JSP tag.
 *
 * @jsp.tag name="simple" 
 *          display-name="SimpleTag" 
 *          body-content="empty" 
 *          description="Simple JSP tag."
 * @jsp.variable name-given="date"
 *               class="java.util.Date"
 *               declare="true"
 *               scope="AT_BEGIN"
 * @jsp.variable name-given="var"
 */
public class SimpleTag extends TagSupport {
    private String action = null;
    private Integer thingy = new Integer(-1);

    /**
     * Sets the action attribute. This is included in the tld file.
     *
     * TODO : check if the @jsp could be on get method or how it was handled in xdoclet1
     *
     * @jsp.attribute
     *     description="The action attribute defines the dummy action to take with this dummy tag."
     *     required="true"
     *     rtexprvalue="true"
     */
    public void setAction(String action) {
        this.action = action;
    }

    /**
     * Just another dummy example that will use default values
     * @jsp.attribute
     *     type=java.lang.Integer
     */
    public void setThingy(Integer thingy) {
        this.thingy = thingy;
    }

    /**
     * Process the start tag for this instance.
     */
    public int doStartTag() throws JspTagException {
        try {
            JspWriter out = pageContext.getOut();
            out.print("Simple tag started with action: " + action);
            pageContext.setAttribute("date", new Date());
            pageContext.setAttribute("var", "<" + thingy + ">");
        } catch (IOException ex) {
            throw new JspTagException("Could not write tag content: " + ex.getMessage());
        }

        return EVAL_BODY_INCLUDE;
    }

    /**
     * Process the end tag. This method will be called on all Tag objects.
     */
    public int doEndTag() throws JspTagException {
        try {
            JspWriter out = pageContext.getOut();
            out.print("Simple tag ended.");
        } catch (IOException ex) {
            throw new JspTagException("Could not write at end tag: " + ex.getMessage());
        }

        return EVAL_PAGE;
    }
}