package org.xdoclet.testapp.web;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpServlet;
import java.io.IOException;

/**
 * Simple Velocity Servlet. This servlet illustrates that your servlet can be any
 * subclass of HttpServlet, as long as all the classes
 * in the hierarchy (and any classes they depend on) are on the classpath.
 *
 * @web.servlet
 *     display-name="Simple Velocity Servlet"
 *     load-on-startup="1"
 *     run-as="TestRole"
 *     name="SimpleVelocityServlet"
 *     description="This is a description for the simple Velocity servlet"
 *
 * @web.servlet-init-param
 *     name="param1"
 *     value="value1"
 *     description="This is a description for the 1st init-param of the simple Velocity servlet"
 *
 * @web.servlet-init-param
 *     name="param2"
 *     value="value2"
 *
 * @web.servlet-mapping
 *     url-pattern="/simple/*"
 *
 * @web.resource-ref
 *     name="jdbc/EmployeeAppDB"
 *     auth="Container"
 *     description="Test resource reference"
 *     type="javax.sql.DataSource"
 *     scope="Shareable"
 *     jndi-name="test jndi??"
 *
 * @web.resource-env-ref
 *     name="jms/StockQueue"
 *     type="javax.jms.Queue"
 *     description="Test resource env ref"
 *
 * @web.env-entry
 *     name="TestEnvName"
 *     type="java.lang.String"
 *     value="testString"
 *     description="a test string for env entry!?"
 *
 * @web.security-role
 *     role-name="TestRole"
 *     description="A description for the test role"
 *
 * @web.security-role
 *     role-name="AnotherRole"
 *
 * @web.security-role-ref
 *     role-name="LinkedTestRole"
 *     role-link="TestRole"
 *     description="A description for the test link role"
 *
 * @web.security-role-ref
 *     role-name="YetAnotherRole"
 *     role-link="AnotherRole"
 *
 * @web.ejb-ref
 *     name="Account"
 *     description="A test reference to the Account EJB"
 *     home="test.interfaces.AccountHome"
 *     link="Account"
 *     remote="test.interfaces.Account"
 *     type="Entity"
 *
 * @web.ejb-local-ref
 *     name="Dummy"
 *     description="A test ref to the local Dummy Session Bean"
 *     home="test.interfaces.DummyLocalHome"
 *     link="Dummy"
 *     local="test.interfaces.DummyLocal"
 *     type="Session"
 * 
 * @jboss.resource-ref
 *     jndi-name="queue/A"
 *     res-ref-name="Queue"
 *
 * @jboss.ejb-ref-jndi
 *     jndi-name="ejb/bank/Account"
 *     ref-name="Account"
 *
 */
public class SimpleVelocityServlet extends HttpServlet {

    /**
     * Called by the server (via the service method) to allow a servlet to handle a POST request.
     * The HTTP POST method allows the client to send data of unlimited length to the Web server
     * a single time and is useful when posting information such as credit card numbers.
     */
    public void doPost(HttpServletRequest request, HttpServletResponse response)
            throws IOException, ServletException {
        // just print Hi!
        response.getWriter().println("Hi!");
    }
}